<?php

namespace App\Service;

use App\Exception;

class ProductService {
    
    private $util;
    private $factory;
    private $combinedConfTela = [];
    private $combinedProducts = [];
    private $combinedChilds = [];

    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ProductFactory $factory) {

        $this->util    = $util;
        $this->factory = $factory;
    }
    
    public function processProducts ($product, $orderData, $generalParams, &$ITCOMANDAVEN_SEQ) {
        $simpleItens = [];
        
        for($x = 0; $x < $product["quantity"]; $x++) {
            if (empty($product["price"]) || ($product["price"] < 0.01 ))
                throw new Exception\ParamsException("O produto: {$product['title']} não atende o preço mínimo por item (R$ 0,01).");
            
            if (empty($product["ref"]))
                throw new Exception\ParamsException("O produto: {$product['title']} não possui código de integração parametrizado.");
                
            $productDetails = $this->factory->getProduct($product["ref"]);
            
            if (empty($productDetails))
                throw new Exception\ParamsException("Não foi encontrado um produto para o código de integração: {$product['ref']} - Produto: {$product['title']}");
                
            $productDetails = array_merge($productDetails, $product);
            
            $productDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ), 6, '0', STR_PAD_LEFT);
            $ITCOMANDAVEN_SEQ++;

            $productDetails["TXPRODCOMVEN"] = " ";
            if (!empty($productDetails["ComplementCategories"]) && count($productDetails["ComplementCategories"]) > 0) {
                foreach ($productDetails["ComplementCategories"] as $obs) {
                    if (strlen($obs['ref']) === 5 && substr($obs['ref'], 2, 1) === '|'){
                        $obsTitle = $this->getObservationName($obs, $generalParams);

                        for ($i = 0 ; $i < $obs["quantity"] ; $i ++)
                            $productDetails["TXPRODCOMVEN"] .= $obsTitle."; ";   
                    }
                }
            }
            
            $productDetails["TXPRODCOMVEN"] .= empty($productDetails["notes"]) ? "" : $productDetails["notes"]."; ";
            
            $productReturn = array(
                "CDFILIAL"          => $generalParams["CDFILIAL"],
                "NRVENDAREST"       => $orderData["NRVENDAREST"],
                "NRCOMANDA"         => $orderData["NRCOMANDA"],
                "NRPRODCOMVEN"      => $productDetails["NRPRODCOMVEN"],
                "CDPRODUTO"         => $productDetails["CDPRODUTO"],
                "QTPRODCOMVEN"      => 1,
                "VRPRECCOMVEN"      => $productDetails["price"], 
                "TXPRODCOMVEN"      => $productDetails["TXPRODCOMVEN"],
                "IDSTPRCOMVEN"      => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                "VRDESCCOMVEN"      => !empty($productDetails["VRDESCCOMVEN"]) ? $productDetails["VRDESCCOMVEN"] : 0,
                "NRLUGARMESA"       => "01",
                "IDPRODIMPFIS"      => "N",
                "CDLOJA"            => $generalParams["CDLOJA"],
                "VRACRCOMVEN"       => 0,
                "NRSEQPRODCOM"      => null,
                "NRSEQPRODCUP"      => $generalParams["NRSEQPRODCUP"],
                "VRPRECCLCOMVEN"    => 0,
                "CDCAIXACOLETOR"    => $generalParams["CDCAIXA"],
                "CDPRODPROMOCAO"    => null,
                "CDVENDEDOR"        => $generalParams["CDVENDEDOR"],
                "NRINSCRCONS"       => preg_replace("/[^0-9]/", "", $orderData["cpfInNote"]),
                "DSOBSPEDDIGCMD"    => $product["notes"],
                "CDSENHAPED"        => isset($orderData['orderNumber']) ? substr($orderData['orderNumber'], 0, 5) : null,
                "ITEM_OBSERVATION"  => [],
            );
            
            $CDPRODPROMO = $productDetails["CDPRODUTO"];
            
            if (!empty($productDetails["ComplementCategories"]) && count($productDetails["ComplementCategories"]) > 0) {
                foreach ($productDetails["ComplementCategories"] as $obs) {
                    if($obs['priceUn'] >= 0.01) {

                        if (strlen($obs['ref']) === 5 && substr($obs['ref'], 2, 1) === '|'){
                            $observationDetails = $this->factory->getObservation($obs["ref"], $generalParams, true);
                        } else{
                            $observationDetails = $this->factory->getProduct($obs["ref"]);
                        }
                        
                        if (empty($observationDetails))
                            throw new Exception\ParamsException("Não foi possivel encontrar o complemento de código: {$obs['ref']}");
                        
                        for($y = 0; $y < $obs['quantity']; $y++){
                            $observationDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ), 6, '0', STR_PAD_LEFT);
                            $ITCOMANDAVEN_SEQ++;
                            
                            $obsReturn = array(
                                "CDFILIAL"       => $generalParams["CDFILIAL"],
                                "NRVENDAREST"    => $orderData["NRVENDAREST"],
                                "NRCOMANDA"      => $orderData["NRCOMANDA"],
                                "NRPRODCOMVEN"   => $observationDetails["NRPRODCOMVEN"],
                                "CDPRODUTO"      => $observationDetails["CDPRODUTO"],
                                "QTPRODCOMVEN"   => 1,
                                "VRPRECCOMVEN"   => $obs['priceUn'], 
                                "TXPRODCOMVEN"   => " ",
                                "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                                "VRDESCCOMVEN"   => 0,
                                "NRLUGARMESA"    => "01",
                                "IDPRODIMPFIS"   => "N",
                                "CDLOJA"         => $generalParams["CDLOJA"],
                                "VRACRCOMVEN"    => 0,
                                "NRSEQPRODCOM"   => $this->util->formatNrSeqProdCom($productDetails["NRPRODCOMVEN"]),
                                "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                                "VRPRECCLCOMVEN" => 0,
                                "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                                "CDPRODPROMOCAO" => $CDPRODPROMO,
                                "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                                "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $orderData["cpfInNote"]),
                                "DSOBSPEDDIGCMD" => "",
                                "CDSENHAPED"     => isset($orderData['orderNumber']) ? substr($orderData['orderNumber'], 0, 5) : null
                            );
                            
                            array_push($simpleItens, $obsReturn); 
                        }
                        
                        if (strlen($obs['ref']) === 5 && substr($obs['ref'], 2, 1) === '|'){
                            array_push($productReturn['ITEM_OBSERVATION'], [
                                'CDFILIAL'        => $generalParams["CDFILIAL"],  
                                'NRVENDAREST'     => $orderData["NRVENDAREST"], 
                                'NRCOMANDA'       => $orderData["NRCOMANDA"], 
                                'NRPRODCOMVEN'    => $productDetails["NRPRODCOMVEN"],  
                                'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],   
                                'CDOCORR'         => $observationDetails['CDOCORR'],  
                                'NRPRODCOMVENOBS' => null, //$observationDetails["NRPRODCOMVEN"],  
                                'QTPRODCOMVENOBS' => $obs['quantity']
                            ]);   
                        }
                    } else{
                        $observationDetails = $this->factory->getObservation($obs["ref"], $generalParams);
                        
                        if (empty($observationDetails))
                            throw new Exception\ParamsException("Não foi possivel encontrar o complemento de código: {$obs['ref']}");
                            
                        array_push($productReturn['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $generalParams["CDFILIAL"],  
                            'NRVENDAREST'     => $orderData["NRVENDAREST"], 
                            'NRCOMANDA'       => $orderData["NRCOMANDA"], 
                            'NRPRODCOMVEN'    => $productDetails["NRPRODCOMVEN"],  
                            'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],   
                            'CDOCORR'         => $observationDetails['CDOCORR'],  
                            'NRPRODCOMVENOBS' => null,  
                            'QTPRODCOMVENOBS' => 1
                        ]);
                    }
                }
            }       
            array_push($simpleItens, $productReturn);
        }    

        return $simpleItens;
    }
    
    public function processCombinedProducts($combinedProduct, $orderData, $generalParams, &$ITCOMANDAVEN_SEQ) {
        $combinedProductsArr = []; $dataCombined = [];
        $cloneProduct = $combinedProduct;
        
        $dataCombined = $this->getCombinedProducts($generalParams['CDFILIAL'], $generalParams['CDCAIXA'], $generalParams['CDLOJA']);
        
        for($x = 0; $x < $combinedProduct["quantity"]; $x++) {
            $massaName    = '';
            $massaPrice   = 0;
            $pizzasPrice  = 0;
            $pizzaflavors = 0;
            $tmpSubItens  = [];
            
            foreach($combinedProduct['combination'] as $tmpItem) {
                for ($j = 0 ; $j < $tmpItem["quantity"]; $j++) {
                    $tmpItem2 = $tmpItem;
                    $tmpItem2["price"] = ($tmpItem["price"] / $tmpItem["quantity"]);
                    $tmpItem2["quantity"] = 1;
                    array_push($tmpSubItens, $tmpItem2);
                }
            }
            $combinedProduct['combination'] = $tmpSubItens;
            
            foreach($combinedProduct['combination'] as &$item) {
                $itemDetails = [];
                
                if(strtoupper(substr($item['title'], 0, 5)) === 'MASSA') {
                    //Encontrar elemento que é a massa. Armazenar seu valor.
                    $massaPrice = $item['price'];
                    $massaName = $item['title'];
                } else {
                    //Somar totalPrice de todos os elementos que não são massa.
                    $pizzasPrice += $item['price'];
                    //encontrar a quantidade com base no numero de elementos removendo a massa.
                    $pizzaflavors += $item['quantity'];
                    
                    // encontrar cdproduto pelo ref do pai (NRBUTTON) e do filho (CDARVPROD)
                    if(isset($dataCombined[$combinedProduct['ref']])) {
                        if(isset($dataCombined[$combinedProduct['ref']]['PRODUCTS'])) {
                            if(isset($dataCombined[$combinedProduct['ref']]['PRODUCTS'][$item['ref']])) {
                                $itemDetails = $dataCombined[$combinedProduct['ref']]['PRODUCTS'][$item['ref']];
                                $item = array_merge($itemDetails, $item);
                            }
                        }
                    }
                    if (empty($itemDetails)) {
                        throw new Exception\ParamsException("
                            Não existe um produto atrelado ao código externo informado.<br>
                            Produto: {$item['title']}.<br>
                            Código externo: {" . $item['ref'] . "}.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
                    }
                }
            }
            
            if(isset($item)) unset($item);
            
            $pizzasPrice += $massaPrice;
            $fakeQuantity = 0;
            
            foreach($combinedProduct['combination'] as &$item) {
                $item['pizzaOddFlavor'] = false;
                
                if(strtoupper(substr($item['title'], 0, 5)) !== 'MASSA') {
                    //gravar nome do elemento massa em todos os sabores como uma observação digitada.
                    $item['observations'] = ' ' . $massaName. '; ';
                    $item['price'] = $pizzasPrice;
                    $item['quantity'] = round(($item['quantity']/$pizzaflavors), 2);
                    $fakeQuantity += $item['quantity'];
                }
            }
            
            if(isset($item)) unset($item);
            
            //no caso de 3 sabores um dos itens deve ser 0.34.
            if($fakeQuantity < 1) {
                $combinedProduct['combination'][1]['quantity'] += 0.01;
                $combinedProduct['combination'][1]['pizzaOddFlavor'] = true;
            }
            
            foreach($combinedProduct['combination'] as $item) {
                if(strtoupper(substr($item['title'], 0, 5)) !== 'MASSA') {
                    $item["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ), 6, '0', STR_PAD_LEFT);
                    $ITCOMANDAVEN_SEQ++;
        
                    $item["TXPRODCOMVEN"] = " ";
                    if (!empty($item["ComplementCategories"]) && count($item["ComplementCategories"]) > 0) {
                        foreach ($item["ComplementCategories"] as $obs) {
                            if (strlen($obs['ref']) === 5 && substr($obs['ref'], 2, 1) === '|'){
                                $obsTitle = $this->getObservationName($obs, $generalParams);
        
                                for ($i = 0 ; $i < $obs["quantity"] ; $i ++)
                                    $item["TXPRODCOMVEN"] .= $obsTitle."; ";   
                            }
                        }
                    }
                    
                    $item["TXPRODCOMVEN"] .= empty($item["notes"]) ? "" : $item["notes"]."; ";
                    
                    $productReturn = array(
                        "CDFILIAL"          => $generalParams["CDFILIAL"],
                        "NRVENDAREST"       => $orderData["NRVENDAREST"],
                        "NRCOMANDA"         => $orderData["NRCOMANDA"],
                        "NRPRODCOMVEN"      => $item["NRPRODCOMVEN"],
                        "CDPRODUTO"         => $item["CDPRODUTO"],
                        "QTPRODCOMVEN"      => $item["quantity"],
                        "VRPRECCOMVEN"      => $item["price"], 
                        "TXPRODCOMVEN"      => $item["TXPRODCOMVEN"],
                        "IDSTPRCOMVEN"      => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                        "VRDESCCOMVEN"      => !empty($item["VRDESCCOMVEN"]) ? $item["VRDESCCOMVEN"] : 0,
                        "NRLUGARMESA"       => "01",
                        "IDPRODIMPFIS"      => "N",
                        "CDLOJA"            => $generalParams["CDLOJA"],
                        "VRACRCOMVEN"       => 0,
                        "NRSEQPRODCOM"      => null,
                        "NRSEQPRODCUP"      => $generalParams["NRSEQPRODCUP"],
                        "VRPRECCLCOMVEN"    => 0,
                        "CDCAIXACOLETOR"    => $generalParams["CDCAIXA"],
                        "CDPRODPROMOCAO"    => null,
                        "CDVENDEDOR"        => $generalParams["CDVENDEDOR"],
                        "NRINSCRCONS"       => preg_replace("/[^0-9]/", "", $orderData["cpfInNote"]),
                        "DSOBSPEDDIGCMD"    => $combinedProduct["notes"],
                        "CDSENHAPED"        => isset($orderData['orderNumber']) ? substr($orderData['orderNumber'], 0, 5) : null,
                        "ITEM_OBSERVATION"  => [],
                        "PIZZAODDFLAVOR"    => $item['pizzaOddFlavor']
                    );
                    
                    $CDPRODPROMO = $item["CDPRODUTO"];
                    
                    if (!empty($item["ComplementCategories"]) && count($item["ComplementCategories"]) > 0) {
                        foreach ($item["ComplementCategories"] as $obs) {
                            if($obs['priceUn'] >= 0.01) {
        
                                if (strlen($obs['ref']) === 5 && substr($obs['ref'], 2, 1) === '|'){
                                    $observationDetails = $this->factory->getObservation($obs["ref"], $generalParams, true);
                                } else{
                                    $observationDetails = $this->factory->getProduct($obs["ref"]);
                                }
                                
                                if (empty($observationDetails))
                                    throw new Exception\ParamsException("Não foi possivel encontrar o complemento de código: {$obs['ref']}");
                                
                                for($y = 0; $y < $obs['quantity']; $y++){
                                    $observationDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ), 6, '0', STR_PAD_LEFT);
                                    $ITCOMANDAVEN_SEQ++;
                                    
                                    $obsReturn = array(
                                        "CDFILIAL"       => $generalParams["CDFILIAL"],
                                        "NRVENDAREST"    => $orderData["NRVENDAREST"],
                                        "NRCOMANDA"      => $orderData["NRCOMANDA"],
                                        "NRPRODCOMVEN"   => $observationDetails["NRPRODCOMVEN"],
                                        "CDPRODUTO"      => $observationDetails["CDPRODUTO"],
                                        "QTPRODCOMVEN"   => 1,
                                        "VRPRECCOMVEN"   => $obs['priceUn'], 
                                        "TXPRODCOMVEN"   => " ",
                                        "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                                        "VRDESCCOMVEN"   => 0,
                                        "NRLUGARMESA"    => "01",
                                        "IDPRODIMPFIS"   => "N",
                                        "CDLOJA"         => $generalParams["CDLOJA"],
                                        "VRACRCOMVEN"    => 0,
                                        "NRSEQPRODCOM"   => $this->util->formatNrSeqProdCom($item["NRPRODCOMVEN"]),
                                        "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                                        "VRPRECCLCOMVEN" => 0,
                                        "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                                        "CDPRODPROMOCAO" => $CDPRODPROMO,
                                        "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                                        "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $orderData["cpfInNote"]),
                                        "DSOBSPEDDIGCMD" => "",
                                        "CDSENHAPED"     => isset($orderData['orderNumber']) ? substr($orderData['orderNumber'], 0, 5) : null
                                    );
                                    
                                    array_push($combinedProductsArr, $obsReturn); 
                                }
                                
                                if (strlen($obs['ref']) === 5 && substr($obs['ref'], 2, 1) === '|'){
                                    array_push($productReturn['ITEM_OBSERVATION'], [
                                        'CDFILIAL'        => $generalParams["CDFILIAL"],  
                                        'NRVENDAREST'     => $orderData["NRVENDAREST"], 
                                        'NRCOMANDA'       => $orderData["NRCOMANDA"], 
                                        'NRPRODCOMVEN'    => $item["NRPRODCOMVEN"],  
                                        'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],   
                                        'CDOCORR'         => $observationDetails['CDOCORR'],  
                                        'NRPRODCOMVENOBS' => null, //$observationDetails["NRPRODCOMVEN"],  
                                        'QTPRODCOMVENOBS' => $obs['quantity']
                                    ]);   
                                }
                            } else{
                                $observationDetails = $this->factory->getObservation($obs["ref"], $generalParams);
                                
                                if (empty($observationDetails))
                                    throw new Exception\ParamsException("Não foi possivel encontrar o complemento de código: {$obs['ref']}");
                                    
                                array_push($productReturn['ITEM_OBSERVATION'], [
                                    'CDFILIAL'        => $generalParams["CDFILIAL"],  
                                    'NRVENDAREST'     => $orderData["NRVENDAREST"], 
                                    'NRCOMANDA'       => $orderData["NRCOMANDA"], 
                                    'NRPRODCOMVEN'    => $item["NRPRODCOMVEN"],  
                                    'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],   
                                    'CDOCORR'         => $observationDetails['CDOCORR'],  
                                    'NRPRODCOMVENOBS' => null,  
                                    'QTPRODCOMVENOBS' => 1
                                ]);
                            }
                        }
                    }
                    
                    array_push($combinedProductsArr, $productReturn);
                }
            }
            $combinedProduct = $cloneProduct;
        }
        return $combinedProductsArr;
    }
    
    private function getCombinedProducts($CDFILIAL, $CDCAIXA, $CDLOJA) {
        if (empty($this->combinedConfTela)) {
            $this->combinedConfTela = $this->factory->getCombinedConfTela($CDFILIAL, $CDCAIXA);
        }
        
        if (empty($this->combinedProducts)) {
            $this->combinedProducts = $this->factory->getCombinedProducts($CDFILIAL, $CDLOJA, $this->combinedConfTela);
        }
        
        if (empty($this->combinedChilds)) {
            $this->combinedChilds = $this->factory->getCombinedChilds($CDFILIAL, $this->combinedConfTela);
        }
        
        $result = [];
        
        foreach ($this->combinedProducts as $combinedProd) {
            // This item serves only as a container. This container holds products that are used to build a combination of products. This is why these products are called combined products.
            $combinedItem = [
                "DSBUTTON"     => $combinedProd['DSBUTTON'], // Button name.
                "PRODUCTS"     => [] // This array will hold the actual combined products, populated below.
            ];

            foreach ($this->combinedChilds as $combinedChild) {
                if (intval($combinedChild['IDTIPOPROD']) == intval($combinedProd['NRBUTTON'])) {
                    $combinedChild['CDFILIAL'] = $CDFILIAL;
                    $combinedChild['CDLOJA']   = $CDLOJA;

                    $combinedItem['PRODUCTS'][$combinedChild['CDARVPROD']] = $combinedChild;
                }
            }
            
            $result[$combinedProd['NRBUTTON']] = $combinedItem;
        }

        return $result;
    }
    
    public function clearCombinedProducts() {
        $this->combinedConfTela = [];
        $this->combinedProducts = [];
        $this->combinedChilds   = [];
    }
    
    public function insertProduct ($product, $generalParams, $orderData) {
        $this->factory->insertProduct($product, $generalParams, $orderData);
    }

    public function insertObservation ($observation, $generalParams, $orderData) {
        $this->factory->insertObservation($observation, $generalParams, $orderData);
    }

    private function getObservationName ($observation, $generalParams) {
         
        if (empty($observation["ref"]))
            throw new Exception\ParamsException("A observação: ".$observation["title"].", não possui código de integração parametrizado.");
        
        $isAcr = false;
        if($observation['priceUn'] >= 0.01){
            $isAcr = true;
        }
        
        $observationDetails = $this->factory->getObservation($observation["ref"], $generalParams, $isAcr);
        
        if (empty($observationDetails))
            throw new Exception\ParamsException("Não foi encontrada ocorrência para o código de integração: ".$observation["ref"]." Complemento - ".$observation["title"], 120);
        
        return $observationDetails["DSOCORR"];
    }
    
    public function shareDiscount($initialDiscount, &$products) {

        $idxToApplyDiscount   = [];
        $totalDiscount        = $initialDiscount;
        $discountLeft         = $initialDiscount;
        $productsTotalValue   = "0.00";
        $discountGivenShare   = "0.00";
        $maxValueItem         = "0.00";
        $maxValueIdx          = -1;
        
        foreach($products as &$product) {
            $product['VRDESCCOMVEN']  = 0;
            $product['discountShare'] = "0.00";
        }
        
        if(isset($product))
            unset($product);

        foreach($products as $i => $product) {
            if((bccomp($product["VRPRECCOMVEN"], "0.01", 2) == 1)) { 
                $productsTotalValue = bcadd($productsTotalValue, $product["VRPRECCOMVEN"], 2);
                array_push($idxToApplyDiscount, $i);
            }
        }

        foreach($idxToApplyDiscount as $idxWithDiscount) {
            $discountShare = bcdiv((bcmul($products[$idxWithDiscount]["VRPRECCOMVEN"], "100.00", 2)), $productsTotalValue, 2);
            $products[$idxWithDiscount]["discountShare"] = $discountShare;
            $discountGivenShare = bcadd($discountGivenShare, $discountShare, 2);
        }

        foreach($products as $i => $product) {
            if(bccomp($product['discountShare'], $maxValueItem, 2) == 1) {
                $maxValueIdx  = $i;
                $maxValueItem = $product['discountShare'];
            }            
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["discountShare"] = bcadd($products[$maxValueIdx]["discountShare"], bcsub("100.00", $discountGivenShare, 2), 2);
        }
    
        foreach($idxToApplyDiscount as $idxWithDiscount) {
            $itemDiscount = bcdiv((bcmul($products[$idxWithDiscount]["discountShare"], $totalDiscount, 2)), "100.00", 2);
            $products[$idxWithDiscount]["VRDESCCOMVEN"] = $itemDiscount;

            $discountLeft = bcsub($discountLeft, $itemDiscount, 2);
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["VRDESCCOMVEN"] = bcadd($products[$maxValueIdx]["VRDESCCOMVEN"], $discountLeft, 2);
            $discountLeft = "0.00";
        }
        
        $leftDiscount = "0.00";
        foreach($products as &$product) {
            if( (bccomp($product['VRDESCCOMVEN'], $product['VRPRECCOMVEN'], 2) == 1) || (bccomp($product['VRDESCCOMVEN'], $product['VRPRECCOMVEN'], 2) == 0) )  {
                $leftDiscount = bcadd($leftDiscount, bcsub($product['VRDESCCOMVEN'], $product['VRPRECCOMVEN'], 2), 2);
                $leftDiscount = bcadd($leftDiscount, "0.01", 2);
                
                $product['VRDESCCOMVEN'] = bcsub($product['VRDESCCOMVEN'], $leftDiscount, 2);
            }
        }
        
        if(isset($product))
            unset($product);

        foreach($products as &$product){
            $maxAddDisc = (bcsub(bcsub($product['VRPRECCOMVEN'], "0.01", 2), $product['VRDESCCOMVEN'], 2));
            if($maxAddDisc >= $leftDiscount) {
                $addDisc = $leftDiscount;
            } else{
                $addDisc = $maxAddDisc;
            }
            
            $product['VRDESCCOMVEN'] = bcadd($product['VRDESCCOMVEN'], $addDisc, 2);
            $leftDiscount = bcsub($leftDiscount, $addDisc, 2);
        }

        if(isset($product))
            unset($product);

        return $products;
    }
    
    
    public function getProductFamilyDetails ($generalParams) {
        return $this->productFactory->getProductFamilyDetails($generalParams);
    }
}